module hdmi_out
(
input                       sys_clk_p,          //system clock positive
input                       sys_clk_n,          //system clock negative 
input                       rst_n,              //reset ,low active
input                       i2c_start_n,
inout                       hdmi_scl,           //HDMI I2C clock
inout                       hdmi_sda,           //HDMI I2C data
output                      hdmi_nreset,        //9134 reset
output                      vout_hs,            //horizontal synchronization for 9134
output                      vout_vs,            //vertical synchronization for 9134
output                      vout_de,            //data valid for 9134
output                      vout_clk,           //clock for 9134
output[23:0]                vout_data           //data for 9134
);
wire                       video_clk;           //pixel clock
wire                       video_hs;            //horizontal synchronization
wire                       video_vs;            //vertical synchronization
wire                       video_de;            //video valid
wire[7:0]                  video_r;             //video red data
wire[7:0]                  video_g;             //video green data
wire[7:0]                  video_b;             //video blue data
wire                       sys_clk;             //single end clock   
wire[9:0]                  lut_index;           //look table index
wire[31:0]                 lut_data;            //look table data
assign vout_hs              = video_hs;
assign vout_vs              = video_vs;
assign vout_de              = video_de;
assign vout_clk             = video_clk;
assign vout_data            = {video_r,video_g,video_b};

wire                        locked;
wire                        done_9134;
wire                         rst_n_dly;
assign hdmi_nreset           = rst_n_dly ;

/*************************************************************************
generate single end clock
**************************************************************************/
GTP_INBUFGDS sys_clk_ibufgds
(
.O                          (sys_clk                  ),
.I                          (sys_clk_p                ),
.IB                         (sys_clk_n                )
);
/*************************************************************************
Generate the pixel clock required for the video
****************************************************************************/
sys_pll video_pll_m0
(
  .clkout0                    (video_clk             ),    // output
  .lock                       (locked                ),          // output
  .clkin1                     (sys_clk               ),      // input
  .rst                        (~rst_n_dly                )             // input

);
/*************************************************************************
Configure the register of 9011 and 9134
****************************************************************************/
wire [15:0] i2c_speed_div;
assign i2c_speed_div = 16'd500;

i2c_config i2c_config_m0(
.rst                        (~rst_n_dly               ),
.clk                        (sys_clk                  ),
.clk_div_cnt                (i2c_speed_div            ),
.restart                    (~i2c_start_n             ),
.i2c_addr_2byte             (1'b0                     ),
.lut_index                  (lut_index                ),
.lut_dev_addr               (lut_data[31:24]          ),
.lut_reg_addr               (lut_data[23:8]           ),
.lut_reg_data               (lut_data[7:0]            ),
.error                      (                         ),
.done                       (done_9134                ),
.i2c_scl                    (hdmi_scl                 ),
.i2c_sda                    (hdmi_sda                 )
);
/*************************************************************************
configure look-up table
****************************************************************************/
lut_hdmi lut_hdmi_m0(
.lut_index                  (lut_index                ),
.lut_data                   (lut_data                 )
);
/*************************************************************************
Call color bar generation module
****************************************************************************/
color_bar color_bar_m0(
.clk                        (video_clk                ),
.rst                        (~done_9134               ),
.hs                         (video_hs                 ),
.vs                         (video_vs                 ),
.de                         (video_de                 ),
.rgb_r                      (video_r                  ),
.rgb_g                      (video_g                  ),
.rgb_b                      (video_b                  )
);

reset_dly reset_dly_u0
(
.clk                        (sys_clk                  ),
.rst_n                      (rst_n                    ),
.rst_n_dly                  (rst_n_dly                )
 );

///`define __nodebug__
`ifndef __nodebug__

wire        hub_tdi;
wire [14:0] hub_tdo;
wire [4:0]  hub_id;
wire        hub_capt;
wire        hub_shift;
wire [14:0] conf_sel;
wire        hub_drck;
wire        i2c_dbg_clk;
reg  [7: 0] rd0;
reg  [7: 0] rd1;

reg[13:0] clk_div;
always @(posedge sys_clk or negedge rst_n_dly)
    if (rst_n_dly==1'b0)
        clk_div <= 14'h0;
    else
        clk_div <= clk_div +1;
`define clk_bit 6

always @(posedge sys_clk) begin
    rd0 <= {3'd0, i2c_start_n, clk_div[`clk_bit+1], hdmi_scl, hdmi_sda, hdmi_nreset};
    rd1 <= rd0;
end

GTP_CLKBUFG clk_buf_I (
    .CLKOUT(i2c_dbg_clk),// OUTPUT  
    .CLKIN(~clk_div[`clk_bit])  // INPUT  
);

hub_I Jtag_I2C(
  .resetn_i(rst_n_dly),      // input
  .drck_o(hub_drck),      // output
  .hub_tdi(hub_tdi),      // output
  .capt_o(hub_capt),      // output
  .shift_o(hub_shift),    // output
  .conf_sel(conf_sel),    // output [14:0]
  .id_o(hub_id),          // output [4: 0]
  .hub_tdo(hub_tdo)       // input  [14:0]
);

deb1 Dbg_I2C(
  .hub_tdi(hub_tdi),      // input
  .hub_tdo(hub_tdo[0]),      // output
  .id_i(hub_id),          // input [4:0]
  .capt_i(hub_capt),      // input
  .shift_i(hub_shift),    // input
  .conf_sel(conf_sel[0]),    // input
  .drck_in(hub_drck),     // input
  .clk(i2c_dbg_clk),              // input
  .resetn_i(rst_n_dly),      // input
  .trig0_i(rd1)            // input [7:0]
);

deb2 Dbg_TMG (
  .hub_tdi(hub_tdi),      // input
  .hub_tdo(hub_tdo[1]),      // output
  .id_i(hub_id),          // input [4:0]
  .capt_i(hub_capt),      // input
  .shift_i(hub_shift),    // input
  .conf_sel(conf_sel[1]),    // input
  .drck_in(hub_drck),     // input
  .clk(sys_clk),              // input
  .resetn_i(rst_n_dly),      // input

  .trig0_i({2'd0, video_r[0], video_g[0], video_b[0], vout_hs, vout_vs, vout_de}),
  .trig1_i(clk_div[13:6]),      // input [7:0]
  .trig2_i(8'h0)       // input [7:0]
);

`endif

endmodule 